@@ -13,7 +13,6 @@
 #include "rapidjson/stringbuffer.h"
 #include <kodi/Filesystem.h>
 #include "epg/ZattooEpgProvider.h"
-#include "epg/EnhancedEpgProvider.h"
 
 #ifdef TARGET_ANDROID
 #include "to_string.h"
@@ -269,12 +268,7 @@ bool ZatData::InitSession(bool isReinit)
   if (m_epgProvider) {
     delete m_epgProvider;
   }
-  if (m_alternativeEpgService) {
-    std::string country = m_serviceRegionCountry.empty() ? m_countryCode : m_serviceRegionCountry;
-    m_epgProvider = new EnhancedEpgProvider(this, *m_epgDB, *m_httpClient, m_categories, m_powerHash, country);
-  } else {
-    m_epgProvider = new ZattooEpgProvider(this, m_providerUrl, *m_epgDB, *m_httpClient, m_categories, m_channelsByUid, m_powerHash);
-  }
+  m_epgProvider = new ZattooEpgProvider(this, m_providerUrl, *m_epgDB, *m_httpClient, m_categories, m_visibleChannelsByCid, m_powerHash);
   return true;
 }
 
@@ -373,7 +367,12 @@ bool ZatData::LoadChannels()
 
   if (!favGroup.channels.empty())
     m_channelGroups.insert(m_channelGroups.end(), favGroup);
-
+  
+  for (const PVRZattooChannelGroup &group : m_channelGroups) {
+    for (const ZatChannel &channel : group.channels) {
+      m_visibleChannelsByCid[channel.cid] = channel;
+    }
+  }
   return true;
 }
 
@@ -385,10 +384,9 @@ PVR_ERROR ZatData::GetChannelGroupsAmount(int& amount)
 
 ZatData::ZatData(KODI_HANDLE instance, const std::string& version,
       const std::string& u, const std::string& p, bool favoritesOnly,
-      bool alternativeEpgService, const STREAM_TYPE& streamType, bool enableDolby, int provider,
+      const STREAM_TYPE& streamType, bool enableDolby, int provider,
       const std::string& parentalPin) :
     kodi::addon::CInstancePVRClient(instance, version),
-    m_alternativeEpgService(alternativeEpgService),
     m_favoritesOnly(favoritesOnly),
     m_enableDolby(enableDolby),
     m_streamType(streamType),
@@ -587,7 +585,7 @@ PVR_ERROR ZatData::GetChannelGroupMembers(const kodi::addon::PVRChannelGroup& gr
 
 PVR_ERROR ZatData::GetChannelsAmount(int& amount)
 {
-  amount = static_cast<int>(m_channelsByCid.size());
+  amount = static_cast<int>(m_visibleChannelsByCid.size());
   return PVR_ERROR_NO_ERROR;
 }
 
@@ -1256,26 +1254,68 @@ PVR_ERROR ZatData::IsEPGTagRecordable(const kodi::addon::PVREPGTag& tag, bool& i
 
 PVR_ERROR ZatData::GetEPGTagStreamProperties(const kodi::addon::PVREPGTag& tag, std::vector<kodi::addon::PVRStreamProperty>& properties)
 {
-  PVR_ERROR ret = PVR_ERROR_FAILED;
   std::ostringstream dataStream;
   ZatChannel channel = m_channelsByUid[tag.GetUniqueChannelId()];
+  
+  std::string url = GetStreamUrlForProgram(channel.cid, tag.GetUniqueBroadcastId(), properties);
+  
+  if (url.empty()) {
+    kodi::Log(ADDON_LOG_WARNING, "Could not get url for channel %s and program %i. Try to get new EPG tag.", channel.cid.c_str(), tag.GetUniqueBroadcastId());
+    time_t referenceTime = (tag.GetStartTime() / 2) + (tag.GetEndTime() / 2);
+    std::ostringstream urlStream;
+    urlStream << m_providerUrl << "/zapi/v3/cached/" + m_powerHash + "/guide"
+        << "?end=" << referenceTime << "&start=" << referenceTime
+        << "&format=json";
+
+    int statusCode;
+    std::string jsonString = m_httpClient->HttpGet(urlStream.str(), statusCode);
+
+    Document doc;
+    doc.Parse(jsonString.c_str());
+    if (doc.GetParseError())
+    {
+      kodi::Log(ADDON_LOG_ERROR, "Loading epg failed at %i", referenceTime);
+      return PVR_ERROR_FAILED;
+    }
+    const Value& channels = doc["channels"];
+    if (!channels.HasMember(channel.cid.c_str())) {
+      kodi::Log(ADDON_LOG_ERROR, "Channel not found in epg.");
+      return PVR_ERROR_FAILED;
+    }
+    const Value& channelEpg = channels[channel.cid.c_str()];
+    if (!channelEpg.IsArray() || channelEpg.GetArray().Empty()) {
+      kodi::Log(ADDON_LOG_ERROR, "Channel has no program at time %i.", referenceTime);
+      return PVR_ERROR_FAILED;
+    }
+    const Value& program = channelEpg.GetArray()[0];
+    int newProgramId = program["id"].GetInt();
+    
+    url = GetStreamUrlForProgram(channel.cid, newProgramId, properties);
+    
+    if (url.empty()) {
+      kodi::Log(ADDON_LOG_ERROR, "Could not get url for channel %s and program %i.", channel.cid.c_str(), newProgramId);
+      return PVR_ERROR_FAILED;
+    }
+  }
+  
+  SetStreamProperties(properties, url);
+  return PVR_ERROR_NO_ERROR;
+}
+
+std::string ZatData::GetStreamUrlForProgram(const std::string& cid, int programId, std::vector<kodi::addon::PVRStreamProperty>& properties)
+{
+  std::ostringstream dataStream;
 
   std::string jsonString;
 
-  kodi::Log(ADDON_LOG_DEBUG, "Get timeshift url for channel %s and program %i", channel.cid.c_str(), tag.GetUniqueBroadcastId());
+  kodi::Log(ADDON_LOG_DEBUG, "Get timeshift url for channel %s and program %i", cid.c_str(), programId);
 
   dataStream << GetStreamParameters();
   dataStream << "&pre_padding=0&post_padding=0";
-  jsonString = HttpPostWithRetry(m_providerUrl + "/zapi/v3/watch/replay/" + channel.cid + "/" + std::to_string(tag.GetUniqueBroadcastId()), dataStream.str());
+  jsonString = HttpPostWithRetry(m_providerUrl + "/zapi/v3/watch/replay/" + cid + "/" + std::to_string(programId), dataStream.str());
 
   std::string strUrl = GetStreamUrl(jsonString, properties);
-  if (!strUrl.empty())
-  {
-    SetStreamProperties(properties, strUrl);
-    ret = PVR_ERROR_NO_ERROR;
-  }
-
-  return ret;
+  return strUrl;
 }
 
 PVR_ERROR ZatData::GetEPGTagEdl(const kodi::addon::PVREPGTag& tag, std::vector<kodi::addon::PVREDLEntry>& edl)
